@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2002-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2002-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -479,7 +479,6 @@ do_handle_info({Proto, _Socket, Data},
   when (Proto =:= tcp) orelse 
        (Proto =:= ssl) orelse 
        (Proto =:= httpc_handler) ->
-
     try Module:Function([Data | Args]) of
 	{ok, Result} ->
 	    handle_http_msg(Result, State); 
@@ -1025,7 +1024,8 @@ handle_response(#state{status = Status0} = State0) when Status0 =/= new ->
            options      = Options,
            profile_name = ProfileName} = State,
     handle_cookies(Headers, Request, Options, ProfileName), 
-    case httpc_response:result({StatusLine, Headers, Body}, Request) of
+    RequestWithIpFamily = add_ipfamily_to_request(Request, Options#options.ipfamily),
+    case httpc_response:result({StatusLine, Headers, Body}, RequestWithIpFamily) of
 	%% 100-continue
 	continue ->
 	    %% Send request body
@@ -1073,6 +1073,22 @@ handle_response(#state{status = Status0} = State0) when Status0 =/= new ->
 	    {stop, normal, NewState}
     end.
 
+sanitize_request_socket_opts(undefined) -> [];
+sanitize_request_socket_opts(Opts) -> Opts.
+
+% Carry over the IP family from the existing state options for the case
+% where we perform a retry or redirect. Without interpolating the ipfamily
+% into the request socket options, the newly started httpc_handler would lose
+% context of the IP family and may try to contact IPv6 hosts over IPv4 via
+% the default ipfamily option "inet". Skip adding it for IPv4 (inet) to
+% prevent TLS errors on following redirects.
+add_ipfamily_to_request(Request, inet) -> Request;
+add_ipfamily_to_request(Request, IpFamily) ->
+    SocketOpts = sanitize_request_socket_opts(Request#request.socket_opts),
+    Request#request{
+        socket_opts = [{ipfamily, IpFamily} | SocketOpts]
+    }.
+
 handle_cookies(_,_, #options{cookies = disabled}, _) ->
     ok;
 %% User wants to verify the cookies before they are stored,
@@ -1299,11 +1315,12 @@ handle_server_closing(State = #state{headers = Headers}) ->
         false -> State
     end.
 
-answer_request(#request{id = RequestId, from = From} = Request, Msg, 
+answer_request(#request{id = RequestId, from = From, request_options = Options} = Request, Msg,
 	       #state{session      = Session, 
 		      timers       = Timers, 
-		      profile_name = ProfileName} = State) -> 
-    httpc_response:send(From, Msg),
+		      profile_name = ProfileName} = State) ->
+    Answer = format_answer(Msg, Options),
+    httpc_response:send(From, Answer),
     RequestTimers = Timers#timers.request_timers,
     TimerRef =
 	proplists:get_value(RequestId, RequestTimers, undefined),
@@ -1710,3 +1727,39 @@ clobber_and_retry(#state{session = #session{id = Id,
         _ ->
             maybe_retry_queue(KeepAlive, State)
     end.
+    
+format_answer(Res, Options) ->
+    FullResult = proplists:get_value(full_result, Options, true),
+    Sync = proplists:get_value(sync, Options, true),
+    do_format_answer(Res, FullResult, Sync).
+do_format_answer({Ref, StatusLine}, _, Sync) when is_atom(StatusLine) ->
+    case Sync of
+        true ->
+            {Ref, {ok, StatusLine}};
+        _ ->
+            {Ref, StatusLine}
+    end;
+do_format_answer({Ref, StatusLine, Headers}, _, Sync) when is_atom(StatusLine) ->
+    case Sync of
+        true ->
+            {Ref, {ok, {StatusLine, Headers}}};
+        _ ->
+            {Ref, StatusLine, Headers}
+    end;
+do_format_answer({Ref, {StatusLine, Headers, BinBody}}, true, Sync) ->
+    case Sync of
+        true ->
+            {Ref, {ok, {StatusLine, Headers, BinBody}}};
+        _ ->
+            {Ref, {StatusLine, Headers, BinBody}}
+    end;
+do_format_answer({Ref, {StatusLine, _, BinBody}}, false, Sync) ->
+    {_, Status, _} = StatusLine,
+    case Sync of
+        true ->
+            {Ref, {ok, {Status, BinBody}}};
+        _ ->
+            {Ref, {Status, BinBody}}
+    end;
+do_format_answer({Ref, {error, _Reason} = Error}, _, _) ->
+    {Ref, Error}.
\ No newline at end of file
