@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -713,7 +713,6 @@ validate_extensions(OtpCert, [], ValidationState =
 		    {ValidationState, UserState0}
 	    end
     end;
-
 validate_extensions(OtpCert,
 		    [#'Extension'{extnID = ?'id-ce-basicConstraints',
 				  extnValue =
@@ -741,13 +740,14 @@ validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-basicConstraints',
     validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
 			SelfSigned, UserState, VerifyFun);
 
+
 validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-keyUsage',
-					   extnValue = KeyUse
+					   extnValue = KeyUses
 					  } | Rest],
 		    #path_validation_state{last_cert=Last} = ValidationState,
 		    ExistBasicCon, SelfSigned,
 		    UserState0, VerifyFun) ->
-    case Last orelse is_valid_key_usage(KeyUse, keyCertSign) of
+    case Last orelse lists:member(keyCertSign, KeyUses) of
 	true ->
 	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState0, VerifyFun);
@@ -757,7 +757,6 @@ validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-keyUsage',
 	    validate_extensions(OtpCert, Rest, ValidationState, ExistBasicCon,
 				SelfSigned, UserState, VerifyFun)
     end;
-
 validate_extensions(OtpCert, [#'Extension'{extnID = ?'id-ce-subjectAltName',
 					   extnValue = Names,
 					   critical = true} = Ext | Rest],
@@ -1480,9 +1479,6 @@ cert_auth_key_id(#'AuthorityKeyIdentifier'{authorityCertIssuer =
 					   SerialNr}) ->
     {ok, {SerialNr, decode_general_name(AuthCertIssuer)}}.
 
-is_valid_key_usage(KeyUse, Use) ->
-    lists:member(Use, KeyUse).
-
 ext_keyusage_includes_any(KeyUse) when is_list(KeyUse) ->
     lists:member(?anyExtendedKeyUsage, KeyUse);
 ext_keyusage_includes_any(_) ->
@@ -1774,8 +1770,8 @@ is_fixed_dh_cert(PublicKeyInfo, Extensions) ->
     case select_extension(?'id-ce-keyUsage', Extensions) of
 	undefined ->
 	    is_dh(Algorithm);
-	#'Extension'{extnValue=KeyUse} ->
-	    is_dh(Algorithm) andalso is_valid_key_usage(KeyUse, keyAgreement)
+	#'Extension'{extnValue = KeyUses} ->
+	    is_dh(Algorithm) andalso lists:member(keyAgreement, KeyUses)
     end.
 	
 is_dh(?'dhpublicnumber')->
@@ -1793,50 +1789,72 @@ is_digitally_sign_cert(OtpCert) ->
 	    lists:member(keyCertSign, KeyUse)
     end.
 
-compatible_ext_key_usage(undefined, _, endentity) -> %% keyusage (first arg )is mandantory in CAs
-     true;
+compatible_ext_key_usage(undefined, _, endentity) ->
+    true;
 compatible_ext_key_usage(_, undefined, _) ->
     true;
-compatible_ext_key_usage(#'Extension'{extnValue = KeyUse}, #'Extension'{extnValue = Purposes}, _) ->
+compatible_ext_key_usage(#'Extension'{extnID = ?'id-ce-keyUsage',
+                                      extnValue = KeyUses},
+                         #'Extension'{extnID = ?'id-ce-extKeyUsage',
+                                      extnValue = Purposes}, Type) ->
     case ext_keyusage_includes_any(Purposes) of
         true ->
             true;
         false ->
-            is_compatible_purposes(KeyUse, Purposes)
+            is_compatible_purposes(KeyUses, Purposes, Type)
     end.
 
-is_compatible_purposes(_, []) ->
+is_compatible_purposes(_, [], _) ->
     true;
-is_compatible_purposes(KeyUse, [?'id-kp-serverAuth'| Rest]) ->
-    (lists:member(digitalSignature, KeyUse) orelse
-     lists:member(keyAgreement, KeyUse)) andalso
-        is_compatible_purposes(KeyUse, Rest);
-is_compatible_purposes(KeyUse, [?'id-kp-clientAuth'| Rest]) ->
-    (lists:member(digitalSignature, KeyUse)
-     orelse
-       (lists:member(keyAgreement, KeyUse) orelse lists:member(keyEncipherment, KeyUse)))
-        andalso is_compatible_purposes(KeyUse, Rest);
-is_compatible_purposes(KeyUse, [?'id-kp-codeSigning'| Rest]) ->
-    lists:member(digitalSignature, KeyUse) andalso
-        is_compatible_purposes(KeyUse, Rest);
-is_compatible_purposes(KeyUse, [?'id-kp-emailProtection'| Rest]) ->
-    ((lists:member(digitalSignature, KeyUse) orelse
-      lists:member(nonRepudiation, KeyUse))
-     orelse
-       (lists:member(keyAgreement, KeyUse) orelse lists:member(keyEncipherment, KeyUse)))
-        andalso is_compatible_purposes(KeyUse, Rest);
-is_compatible_purposes(KeyUse, [Id| Rest]) when Id == ?'id-kp-timeStamping';
-                                                Id == ?'id-kp-OCSPSigning'->
-    (lists:member(digitalSignature, KeyUse) orelse
-     lists:member(nonRepudiation, KeyUse)) andalso
-        is_compatible_purposes(KeyUse, Rest);
-is_compatible_purposes(KeyUse, [_| Rest]) -> %% Unknown purposes are for user verify_fun to care about
-    is_compatible_purposes(KeyUse, Rest).
+is_compatible_purposes(KeyUses, [?'id-kp-serverAuth'| Rest], ca = Type) ->
+    %% keyCertSign is already verified for a ca and considered compatible
+    is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-serverAuth'| Rest], endentity = Type) ->
+    IsServerAuthComp = case lists:member(digitalSignature, KeyUses) of
+                           true ->
+                               true;
+                           false ->
+                               lists:member(keyAgreement, KeyUses)  orelse
+                                   lists:member(keyEncipherment, KeyUses)
+                       end,
+    IsServerAuthComp andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-clientAuth'| Rest], ca = Type) ->
+    %% keyCertSign is already verified for a ca and considered compatible
+    is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-clientAuth'| Rest], endentity = Type) ->
+    IsClientAuthComp = case lists:member(digitalSignature, KeyUses) of
+                           true ->
+                               true;
+                           false ->
+                               lists:member(keyAgreement, KeyUses)
+                       end,
+    IsClientAuthComp andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-codeSigning'| Rest], Type) ->
+    lists:member(digitalSignature, KeyUses) andalso
+        is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [?'id-kp-emailProtection'| Rest], Type) ->
+    IsEmailProtCompatible = case (lists:member(digitalSignature, KeyUses) orelse
+                                  lists:member(nonRepudiation, KeyUses)) of
+                                true ->
+                                    true;
+                                false ->
+                                    lists:member(keyAgreement, KeyUses) orelse
+                                        lists:member(keyEncipherment, KeyUses)
+                            end,
+    IsEmailProtCompatible andalso is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [Id| Rest],Type) when Id == ?'id-kp-timeStamping';
+                                                      Id == ?'id-kp-OCSPSigning'->
+    (lists:member(digitalSignature, KeyUses) orelse
+     lists:member(nonRepudiation, KeyUses)) andalso
+        is_compatible_purposes(KeyUses, Rest, Type);
+is_compatible_purposes(KeyUses, [_| Rest], Type) -> %% Unknown purposes are for user verify_fun to care about
+    is_compatible_purposes(KeyUses, Rest, Type).
 
-ca_known_extend_key_use(ExtKeyUse) ->
+
+ca_known_extend_key_use(ExtKeyUses) ->
     CAExtSet = ca_known_ext_key_usage(),
-    Intersertion = sets:intersection(CAExtSet, sets:from_list(ExtKeyUse)),
-    not sets:is_empty(Intersertion).
+    Intersection = sets:intersection(CAExtSet, sets:from_list(ExtKeyUses)),
+    not sets:is_empty(Intersection).
 
 ca_known_ext_key_usage() ->
     %% Following extended key usages are known
