@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -251,7 +251,12 @@ format_error({warning,Term}) ->
 format_error(ftr_after_prefix) ->
     "feature directive not allowed after exports or record definitions";
 format_error(tqstring) ->
-    "Triple-quoted (or more) strings will change meaning in OTP-27.0";
+    "triple-quoted (or more) strings will change meaning in OTP-27.0";
+format_error(string_concat) ->
+    "adjacent string literals without intervening white space\n"
+        "In OTP-27.0 this will be a triple-quoted string or an error.\n"
+        "Rewrite them as one string, or insert white space\n"
+        "between the strings.";
 format_error(E) -> file:format_error(E).
 
 -spec scan_file(FileName, Options) ->
@@ -352,9 +357,10 @@ parse_file(Epp) ->
     case epp_request(Epp, scan_erl_form) of
 	{ok,Toks} ->
             Warnings =
-                [{warning, {erl_anno:location(Anno),?MODULE,tqstring}}
-                 %% Warn about using 3 or more double qoutes
-                 || {tqstring,Anno,_} <- Toks],
+                [{warning, {erl_anno:location(Anno),?MODULE,Tag}}
+                 || {Tag,Anno,_} <- Toks,
+                    %% Warn for string concatenation without white space
+                    Tag =:= string_concat],
             case erl_parse:parse_form(Toks) of
                 {ok, Form} ->
                     [Form|Warnings] ++ parse_file(Epp);
@@ -1801,6 +1807,8 @@ macro_arg([{'fun',Lc}|[{'(',_}|_]=Toks], E, Arg) ->
     macro_arg(Toks, ['end'|E], [{'fun',Lc}|Arg]);
 macro_arg([{'fun',_}=Fun,{var,_,_}=Name|[{'(',_}|_]=Toks], E, Arg) ->
     macro_arg(Toks, ['end'|E], [Name,Fun|Arg]);
+macro_arg([{'maybe',Lb}|Toks], E, Arg) ->
+    macro_arg(Toks, ['end'|E], [{'maybe',Lb}|Arg]);
 macro_arg([{'receive',Lr}|Toks], E, Arg) ->
     macro_arg(Toks, ['end'|E], [{'receive',Lr}|Arg]);
 macro_arg([{'try',Lr}|Toks], E, Arg) ->
